package edu.berkeley.sbp;
import edu.berkeley.sbp.util.*;
import edu.berkeley.sbp.*;
import edu.berkeley.sbp.*;
import java.io.*;
import java.util.*;
import java.lang.reflect.*;
import java.lang.ref.*;

/** currently this class exports only static methods to create repetitions; there are no public instance methods or constructors */
public class Repeat extends Union {

    /** repeat zero or one times */
    public  static Repeat maybe(Element e)              { return new Repeat(e, true, false); }
    /** repeat zero or more times */
    public  static Repeat many0(Element e)              { return new Repeat(e, true, true); }
    /** repeat zero or more times, separated by <tt>sep</tt> */
    public  static Repeat many0(Element e, Element sep) { return new Repeat(e, true, true, sep); }
    /** repeat one or more times */
    public  static Repeat many1(Element e)              { return new Repeat(e, false, true); }
    /** repeat one or more times, separated by <tt>sep</tt> */
    public  static Repeat many1(Element e, Element sep) { return new Repeat(e, false, true, sep); }
    /** repeat zero or more times, matching a maximal sequence of atoms */
    public  static Repeat maximal0(Element e)              { return new Repeat(e, true, true, null, true); }
    /** repeat one or more times, matching a maximal sequence of atoms */
    public  static Repeat maximal1(Element e)              { return new Repeat(e, false, true, null, true); }
    /** repeat one or more times, separated by <tt>sep</tt>, matching a maximal sequence of atoms */
    public  static Repeat maximal1(Element e, Element sep) { return new Repeat(e, false, true, sep, true); }

    // Instance //////////////////////////////////////////////////////////////////////////////
    
    final boolean zeroOkay, manyOkay;
    final Element e;
    final Element separator;
    final boolean maximal;

    Repeat(final Element e, boolean zeroOkay, boolean manyOkay) { this(e, zeroOkay, manyOkay, null); }
    Repeat(final Element e, boolean zeroOkay, boolean manyOkay, Element separator) { this(e, zeroOkay, manyOkay, separator, false); }
    Repeat(final Element e, boolean zeroOkay, boolean manyOkay, final Element separator, boolean maximal) {
        super(e+(!manyOkay ? "?" : (zeroOkay ? "*" : "+"))+(separator==null?"":("/"+separator.toString())), true);
        this.e = e;
        this.maximal = maximal;
        this.zeroOkay = zeroOkay;
        this.manyOkay = manyOkay;
        this.separator = separator;
        Union who = this;
        if (maximal) {
            who = new Union(this+"++");
            add(new Sequence.Singleton(who, null, null) {
                    public Topology noFollow() { return separator==null ? e.toAtom() : separator.toAtom(); }
                });
        }
        if (zeroOkay) {
            who.add(new Sequence.Constant.Empty());
            if (manyOkay) who.add(new Sequence.Singleton(many1(e, separator), null, null));
            else          who.add(new Sequence.Singleton(e, null, null));
        } else {
            who.add(new Sequence.RewritingSequence(null, new Element[] { e }, null, null));
            if (this.separator==null) {
                who.add(new Sequence.Unwrap(new Element[] { e,                 Repeat.this }, null, null));
            } else {
                who.add(new Sequence.Unwrap(new Element[] { e, this.separator, Repeat.this }, new boolean[] { false, true, false }, null, null));
            }
        }
    }
}
